@@ -1,4 +1,4 @@
-"""gustaf/gustaf/utils/tictoc.py
+"""gustaf/gustaf/utils/tictoc.py.
 
 Timer that tics, tocs and logs.
 """
